/// GraphicsTown2006 - CS559 Sample Code
// written by Michael Gleicher - originally written Fall, 2000
// updated Fall, 2005
//
#include "GrTown_PCH.H"

#include "GrWorld.H"
#include "DrawingState.H"
#include "DrawUtils.H"
#include "GrObject.h"

// stuff in the world
GrObject* theObjects = 0;

// things for drawing the world

// remember, these function have some responsibilities defined in the
// header... 
// for now these do simple things. in the future, maybe they will draw
// more nicely
void drawSky(DrawingState* st)
{
	
	glEnable(GL_NORMALIZE);
	float f = 0, a = 0.2;
	// figure out the sky color...
	if (st->timeOfDay < 4) a = 0.6; 
	else if (st->timeOfDay < 8) a = 0.6 - (st->timeOfDay - 4) * 0.15; 
	else if (st->timeOfDay < 16) a = 0; 
	else if (st->timeOfDay < 20) a = 0.6 - (20 - st->timeOfDay) * 0.15;
	else a = 0.6;

	if (st->timeOfDay < 10) 
		f = 0.08 * st->timeOfDay;		
	else if (st->timeOfDay < 18) 
		f = 0.08 * (20 - st->timeOfDay);
	else f = 0;

	glClearColor(st->sky.r,st->sky.g,st->sky.b,st->sky.a);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT /*| GL_STENCIL_BUFFER_BIT*/);

	glPushAttrib(GL_ENABLE_BIT | GL_DEPTH_BUFFER_BIT);
	Color c(0.8, 0.7, 0.6);
	glDisable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glColor(c);


	float tx = 0, ty = 0, tz = 0;
	Matrix m;
	st->camera->getCamera(m);
	invertMatrix(m);
	transformMatrix(m, tx, ty, tz, tx, ty, tz);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(tx, ty, tz);


	square(5, 5, UP_Z, "front.bmp");
	square(5, -5, DOWN_X, "left.bmp");
	square(5, 5, UP_X, "right.bmp");
	square(5, -5, DOWN_Y, "top.bmp");
	square(5, -5, DOWN_Z, "back.bmp");
	square(5, 5, UP_Y, "top.bmp");

	csquare(3, 3, UP_Z, 0.5, 0.5, 0.5, f);
	csquare(3, -3, DOWN_X, 0.5, 0.5, 0.5, f);
	csquare(3, 3, UP_X, 0.5, 0.5, 0.5, f);
	csquare(3, -3, DOWN_Y, 0.5, 0.5, 0.5, f);
	csquare(3, -3, DOWN_Z, 0.5, 0.5, 0.5, f);
	csquare(3, 3, UP_Y, 0.5, 0.5, 0.5, f);

	csquare(2, 2, UP_Z, 0, 0, 0, a);
	csquare(2, -2, DOWN_X, 0, 0, 0, a);
	csquare(2, 2, UP_X, 0, 0, 0, a);
	csquare(2, -2, DOWN_Y, 0, 0, 0, a);
	csquare(2, -2, DOWN_Z, 0, 0, 0, a);
	csquare(2, 2, UP_Y, 0, 0, 0, a);

	



	glClear(GL_DEPTH_BUFFER_BIT);
	glPopAttrib();
	glPopMatrix();




}

// for now, just draw a huge green square. 10 miles on a side (5000
// ft/mile) 
void drawEarth(DrawingState* st)
{
	cfg.texture("grass.jpg");
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	st->ground(0,84,24);

	glHint(GL_FOG_HINT, GL_NICEST);
	glEnable(GL_POLYGON_OFFSET_FILL);
	glPolygonOffset(1.,10);
	glBegin(GL_POLYGON);
	glNormal3f(0,1,0);
	glColor4fv(&st->ground.r);
	glTexCoord2d(2000, 0);
	glVertex3d(-25000,0, 25000);
	glTexCoord2d(2000, 2000);
	glVertex3d( 25000,0, 25000);
	glTexCoord2d(0, 2000);
	glVertex3d( 25000,0,-25000);
	glTexCoord2d(0, 0);
	glVertex3d(-25000,0,-25000);
	glEnd();
	glDisable(GL_POLYGON_OFFSET_FILL);
	glHint(GL_FOG_HINT, GL_DONT_CARE);

}

// 
// setup lighting
void setupLights(DrawingState* dr)
{
	// depending on time of day, the lighting changes
	// ambient is either night or day
	if ((dr->timeOfDay >= 5) && (dr->timeOfDay <=19)) {
		float a0[] = {.4f,.4f,.4f,.4f};
		glLightfv(GL_LIGHT0, GL_AMBIENT, a0);
	} else {
		float a1[] = {.2f,.2f,.2f,.2f};
		glLightfv(GL_LIGHT0, GL_AMBIENT, a1);
	}
	// directional, if its on, depends on what hour it is
	float pos[4] = {0,0,0,0};
	if ((dr->timeOfDay >= 5) && (dr->timeOfDay <=19)) {
		float angle = (((float)(dr->timeOfDay-5)) / 7.f) * (3.14159f/2.f);
		pos[0] = (float) cos(angle);
		pos[1] = (float) sin(angle);
	}
	else {
		pos[1] = -1;
	}
	glLightfv(GL_LIGHT0, GL_POSITION, pos);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
}

// $Header: /p/course/cs559-gleicher/private/CVS/GrTown/GrWorld.cpp,v 1.2 2007/11/01 21:25:09 gleicher Exp $
