/// GraphicsTown2006 - CS559 Sample Code
// written by Michael Gleicher - originally written Fall, 2000
//
// updated Fall 2006 - more testing code

//
#include "glee.h"
#include "grave.h"
#include "global.h"

#include "GrTown_PCH.H"
#include "GraphicsTownUI.H"
#include "FlyCamera.H"
#include "GrWorld.H"
#include "DrawUtils.H"
#include "Texture.H"
#include "Examples/Objects.H"
#include "Examples/SimpleBehaviors.H"
#include "Examples/Cars.H"

// Example code (objects and behaviors)
#include "Examples/Suburbs.H"

#include "Roads/Roads.H"
#include "Roads/Drive.H"

// define this to get 2 cars that always turn
// #define TESTCARS

#include "bird.h"
#include "tree.h"
#include "fireworks.h"
#include "casper.h"
#include "widow.h"
#include "depthSorter.h"


#include <ctime>
#include <cmath>

const int numCars = 100;
const int nGrids = 5;

depthSorter sorter;

void sorterInsert(special* sp) {
	add(sp);
	sorter.insert(sp);
}

int main(int /*argc*/, char** /*argv*/)
{ 
	using namespace configure;
	cfg.setup();
	srand(time(0));
	// put in some texture paths - look all over the place
	texturePaths.push_back("Textures/Textures");
	texturePaths.push_back("Textures/signs");
	texturePaths.push_back("Textures/Objects");
	texturePaths.push_back(".");
	texturePaths.push_back("..");
	
	texturePaths.push_back("../Textures/signs");
	
	texturePaths.push_back("../Textures/Textures");
	
	texturePaths.push_back("../Textures/Objects");
	//texturePaths.push_back("c:/src/GraphicsTown/Textures/signs");
	//texturePaths.push_back("c:/src/GraphicsTown/Textures/Textures");


	// add some more stuff
	GrObject* o1 = new Church;
	o1->interesting = true;
	o1->laX = 0; o1->laY = 0; o1->laZ = 0;
	o1->lfX = -50; o1->lfY = 100; o1->lfZ = 300;
	add(o1,-100,0,100,pi/2.f);


	// *****************************************************************
	//  Make your town here

	for (int i = 0; i < numBirds; ++i) {
		add(new bird(drand(9, 11)));
	}

	

	
	vector<region> regions;
	vector<region> reg_graves;
	vector<region> widow_regions;
	regions.push_back(region(0, 0, -300, 200));
	reg_graves.push_back(region(0, 0, -300, 200));
	widow_regions.push_back(region(0, 0, -300, 200));
	
	for (int i = 0; i < numTrees; ++i) {
		double x = drand(-600, 200);
		double y = drand(-250, 400);
		bool ok = true;
		for (int j = 0; j < regions.size(); ++j)
			if (regions[j].in(x, y)) ok = false;
		if (!ok) {--i; continue; }
		regions.push_back(region(x - 70, y - 70, x + 70, y + 70));
		reg_graves.push_back(region(x - 20, y - 20, x + 20, y + 20));
		widow_regions.push_back(region(x - 10, y - 10, x + 10, y + 10));
		add(tree::generate(x, y));
	}
	GrObject* fw = new fireworks(750, 0, 1500);
	add(fw);

	for (int i = 0; i < numGraves; ++i) {
		double x = drand(-600, 0);
		double y = drand(-400, 500);
		bool ok = true;
		for (int j = 0; j < reg_graves.size(); ++j)
			if (reg_graves[j].in(x, y) || reg_graves[j].in(x + 60, y)) ok = false;
		if (!ok) {--i; continue; }
		reg_graves.push_back(region(x - 30, y - 30, x + 60, y + 30));
		
		float size = drand(2.0, 2.5);
		add(new grave(x, 0, y, 90, size * drand(0.8, 1.2), size * drand(0.8, 1.2), size * drand(0.8, 1.2)));
		widow_regions.push_back(region(x - 7 * size, y - 7 * size, x + 7 * size, y + 7 * size));
	}
	
	
	sorterInsert(new casper(100,0,100));
	sorterInsert(new casper(-100,0,0));
	sorterInsert(new casper());
	
	widow* girl = new widow(&widow_regions);
	girl->name = "widow";
	girl->laX = 0; girl->laY = 0; girl->laZ = 0;
	girl->lfX = 70; girl->lfY = 50; girl->lfZ = 0;
	girl->interesting = true;
	sorterInsert(girl);
	add(&sorter);
	

	// first, some really simple things
/*
	// cubes are particularly easy since they set their own position
	// we need to raise the cube since we're giving the position of the
	// center 
	GrObject* cube1 = new Cube(0,5,100,10,   .7f, .6f, .3f);
	add(cube1);
	cube1->name = "Cube1";
	// make it an interesting object (so its easy to look at)
	// and give a place to look at it from
	// this is a good object to look at since it won't move
	cube1->interesting = true;
	cube1->laX = 0; cube1->laY = 0; cube1->laZ = 0;
	cube1->lfX = 70; cube1->lfY = 50; cube1->lfZ = 0;

	// make another cube - make this one spin around
	GrObject* cube2 = new Cube(100,5,0,10,   .3f, .6f, .7f);
	add(cube2);
	new Spin(cube2);

	// now to demonstrate hierarchy - make another cube that spins, and
	// put a smaller cube on top of it
	GrObject* cube3 = new Cube(0,5,0,10,   .7f, .3f, .6f);
	add(cube3);
	new Spin(cube3,.001f);
	GrObject* cube4 = new Cube(0,7.5,0, 5, 1,0,1);
	cube3->add(cube4);
*/

	/*
	////////////////////////////////////////////////////////////////////////
	// now to make a real town!
	int r,c;
	// make a 5x5 grid of town blocks - 5 houses per
	for( r=0; r<5; r++) {
	for( c=0; c<5; c++) {
	add(new SimpleSubdivision(5),static_cast<float>(r*530),0,static_cast<float>(c*230));
	add(new StraightRoad(static_cast<float>(r*530),static_cast<float>(c*230),static_cast<float>(r*530+500),static_cast<float>(c*230)));
	}
	}
	// make cross streets
	for(int r=0; r<=5; r++) {
	for(c=0; c<4; c++) {
	add(new StraightRoad(static_cast<float>(r*530 - 15), static_cast<float>(c*230 + 15), static_cast<float>(r*530 - 15), static_cast<float>(c*230+215)));
	}
	}

	// make intersections
	// make an intersection intersesting so we can look at it
	for(int r=0; r<=5; r++) {
	for(c=0; c<5; c++) {
	GrObject* g = new Intersection(static_cast<float>(r*530-15), static_cast<float>(c*230));
	if ( (r==2) && (c==3) ) {
	g->interesting = 1;
	g->name = "Intersection(2,3)";
	g->laX = static_cast<float>(r*530-15);    g->laY = 0;    g->laZ = static_cast<float>(c*230);
	g->lfX = static_cast<float>(r*530+25);   g->lfY = 100;   g->lfZ = static_cast<float>(c*230+150);
	}
	add(g);
	}
	}

	#ifndef TESTCARS
	// add some cars
	for(int r=0; r<50; r++) {
	Car* c;
	switch(rand() % 3) {
	case 0: c = new Van(rand()); break;
	case 1: c = new SUV(rand()); break;
	case 2: c = new HatchBack(rand()); break;
	}
	add(c);
	new RandomDrive(c,theRoads[rand() % theRoads.size()],.2f,rand() % 2);
	}
	#endif

	#ifdef TESTCARS
	// two cars - one always turns right, one always turns left
	Car* c1 = new HatchBack(1);
	Car* c2 = new HatchBack(2);
	add(c1);
	add(c2);
	new RandomDrive(c1,theRoads[7],.5,0,-1);
	new RandomDrive(c2,theRoads[8],.5,0,1);

	#endif


	// a race track
	Road* t = new RoundRoad(-250,250,100);
	add(t);
	t->name = "Track";
	t->interesting = true;
	t->laX = -250; t->laY = 0;   t->laZ = 250;
	t->lfX = -600; t->lfY = 200; t->lfZ = 450;
	// make cars go around the track
	Car* h = new HatchBack(1);
	h->name="Race1";		// warning! we can only do this since we don't delete
	add(h);
	new SimpleDrive(h,t,0,0);
	h = new HatchBack(3);
	h->name="Race2";		// warning! we can only do this since we don't delete
	add(h);
	Drive* d = new SimpleDrive(h,t,0,1);
	d->speed *= 2;
	*/

	// *****************************************************************
	// now make a UI
	FlyCamera* fc = new FlyCamera;
	Map* m = new Map;

	add(fc);
	add(m);

	GraphicsTownUI grTown;
	grTown.make_window();
	grTown.townView->defaultCamera = fc;
	grTown.window->show();
	cfg.ui = &grTown;
	Fl::run();
	return 1;
}

// $Header: /p/course/cs559-gleicher/private/CVS/GrTown/Main.cpp,v 1.3 2007/11/11 00:29:13 gleicher Exp $
