#pragma once

#include <vector>

#include "../gl.h"

class triple {
public:
		triple() : x(0), y(0), z(1) {}
		triple(double ix, double iy, double iz) : x(ix), y(iy), z(iz) {}
		triple(const double arr[3]) : x(arr[0]), y(arr[1]), z(arr[2]) {}
		triple operator+ (const triple& t) {return triple(x + t.x, y + t.y, z + t.z);}
		triple operator- (const triple& t) {return triple(x - t.x, y - t.y, z - t.z);}
		triple operator* (const double  s) {return triple(x * s, y * s, z * s);}
		
		double x;
		double y;
		double z;

		void normalize();

		triple rotate(triple axis, double angle);

		double dot(triple b) {return x * b.x + y * b.y + z * b.z;}

		static triple interpolate(double, triple, triple);
		triple cross(triple a);

		void draw() {glVertex3d(x,y,z);}

		double& operator[] (unsigned i) {switch(i){ case 0: return x; case 1: return y; case 2: return z; default: throw 0;}}
};

class Point
{
	
private:
	triple pos;
	triple norm;
	triple dir;
	
public:

	static triple cross(triple a, triple b);
	Point(void);
	~Point(void);

	static Point interpolate(double a, Point p1, Point p2) {return Point(triple::interpolate(a, p1.pos, p2.pos), triple::interpolate(a, p1.norm, p2.norm), triple::interpolate(a, p1.dir, p2.dir));}

	Point(triple p, triple n, triple d) : pos(p), norm(n), dir(d) {}
	Point(double px, double py, double pz, double nx, double ny, double nz, double dx, double dy, double dz) : pos(px, py, pz), norm(nx, ny, nz), dir(dx, dy, dz) {}

	void position(double arr[3]) {pos = arr;}
	void position(double x, double y, double z) {pos = triple(x,y,z);}
	triple position() {return pos;}


	void normal(double arr[3]) {norm = arr;}
	void normal(double x, double y, double z) {norm = triple(x,y,z);}
	triple normal() {return norm;}

	void direction(double arr[3]) {dir = arr;}
	void direction(double x, double y, double z) {dir = triple(x,y,z);}
	triple direction() {return dir;}

	void normalize_direction();
	void normalize_normal();
	void normalize() {normalize_direction(); normalize_normal();}

	std::pair<triple, triple> up_out();
	triple out() {triple o = dir.rotate(norm, 90); o.normalize(); return o;}
};



triple operator* (const double  s, const triple& t);