#include "bird.h"
#include "global.h"
#include "DrawingState.h"
#include "texture.h"

using namespace bird_config;

vector<bird*> bird::flock;
int bird::count = 1;
string bird::name = "bird0";


bool white = false;

Texture* body[5];
Texture* wing;

void init() {
	static bool made= false;
	body[0] = fetchTexture("bird-bottom.tga");
	body[1] = fetchTexture("bird-top.tga");
	body[2] = fetchTexture("bird-front.tga");
	body[3] = fetchTexture("bird-back.tga");
	body[4] = fetchTexture("bird-side.tga");

	wing = fetchTexture("bird-wing.tga");


	made = true;
}

void drawbody() {
	glScalef(0.5, 0.5, 0.5);
	if (!white)
		body[4]->bind();
	polygoni(4, 
		-1, -1, -1, 
		1, -1, -1, 
		1, 1, -1, 
		-1, 1, -1);

	polygoni(4, 
		-1, -1, 1, 
		-1, 1, 1, 
		1, 1, 1, 
		1, -1, 1);
	if (!white)
		body[2]->bind();	
	polygoni(4, 
		1, -1, 1, 
		1, 1, 1, 
		1, 1, -1, 
		1, -1, -1);
	if (!white)
		body[3]->bind();
	polygoni(4, 
		-1, -1, 1, 
		-1, -1, -1, 
		-1, 1, -1, 
		-1, 1, 1);
	if (!white)
		body[1]->bind();
	polygoni(4,
		1, 1, 1,
		-1, 1, 1,
		-1, 1, -1,
		1, 1, -1);
	if (!white)
		body[0]->bind();
	polygoni(4,
		-1, -1, -1,
		-1, -1, 1,
		1, -1, 1,
		1, -1, -1);

}

void drawwing() {
	glScalef(0.5, 0.5, 0.5);
	if (!white)
		wing->bind();
	polygoni(4, 
		-1, -1, -1, 
		1, -1, -1, 
		1, 1, -1, 
		-1, 1, -1);

	polygoni(4, 
		-1, -1, 1, 
		-1, 1, 1, 
		1, 1, 1, 
		1, -1, 1);	
	polygoni(4, 
		1, -1, 1, 
		1, 1, 1, 
		1, 1, -1, 
		1, -1, -1);
	polygoni(4, 
		-1, -1, 1, 
		-1, -1, -1, 
		-1, 1, -1, 
		-1, 1, 1);
	polygoni(4,
		1, 1, 1,
		-1, 1, 1,
		-1, 1, -1,
		1, 1, -1);
	polygoni(4,
		-1, -1, -1,
		-1, -1, 1,
		1, -1, 1,
		1, -1, -1);
}

void bird::draw(DrawingState* d) {
	init();
	white = false;

	glColor3f(1, 1, 1);
	if (string(d->camera->name).substr(0, 3) == "Map") {white = true; glBindTexture(GL_TEXTURE_2D, 0);}
	glTranslatef(x, z, y);
	if (dx != 0)
		glRotatef(atan2(dx, dy) / 3.14159 * 180, 0, 1, 0);
	glScalef(size, size, size);
	glRotatef(-90, 0, 1, 0);
	glPushMatrix();
	glScalef(1.1, 0.2, 0.2);
	drawbody();
	glPopMatrix();
	glPushMatrix();		
	glRotatef(angle1, 1, 0, 0);
	glTranslatef(-0.1, 0, 0.6);
	glScalef(0.75, 0.1, 1);
	drawwing();
	glPopMatrix();
	glPushMatrix();
	glRotatef(angle1, -1, 0, 0);
	glTranslatef(-0.1, 0, -0.6);
	glScalef(0.75, 0.1, 1);
	drawwing();
	glPopMatrix();
	glEnable(GL_TEXTURE);

}

void bird::simulateUntil(unsigned long time) {
	long el = 0;
	if (last == 0) last = time;
	angle1 += (el = (time - last)) * 0.07; 

	// little trick, since wings are identical, we don't need to go up and down, just skip
	// some of the rotation, that is, each wing is just rotating in a circle rather than
	// flapping...
	if (angle1 > 40 && angle1 < 140) angle1 += 100;
	if (angle1 > 220) angle1 -= 260;
	last = time;

	float num = 0;
	float numa = 0;
	float ndx = 0;
	float ndy = 0;
	float avx = 0;
	float avy = 0;
	float posx = 0;
	float posy = 0;

	// ok, this is ugly, calculate some values for the birds in the flock.
	for(vector<bird*>::iterator i = flock.begin(); i != flock.end(); ++i) {
		bird* b = *i;
		if (b == this) continue;

		float d = (x - b->x) * (x - b->x) + (y - b->y) * (y - b->y);

		// if nearby
		if (d > 1 && d < viewDepth) {
			float n[3], bn[3];
			n[2] = bn[2] = 0;
			n[0] = x;
			bn[0] = b->x;
			n[1] = y;
			bn[1] = b->y;
			normalize(n);
			normalize(bn);
			float a = acos(n[0] * bn[0] + n[1] * bn[1]);
			// if visible
			if (a < viewAngle) {
				num++;
				ndx += b->dx;
				ndy += b->dy;
				posx += b->x;
				posy += b->y;
				if (d < avoidDepth) {
					numa++;
					avx += (x - b->x)/d;
					avy += (y - b->y)/d;
				}
			}
		}
	}
	// change to averages
	float av = 1 / num;
	ndx *= av;
	ndy *= av;
	posx *= av;
	posy *= av;
	posx -= x;
	posy -= y;
	avx /= numa;
	avy /= numa;
	float mag;
	
	// normalize stuff... why do I do this? I don't know
	if (num != 0) {
		mag = ndx * ndx + ndy * ndy;
		mag = sqrt(mag);
		ndx /= mag;
		ndy /= mag;

		mag = posx * posx + posx * posy;

		if (mag <= 0) posx = posy = 0;
		else {
			mag = sqrt(mag);
			posx /= mag;
			posy /= mag;
		}
	} else { posx = posy = ndx = ndy = 0; }

	if (numa != 0) {
		mag = avx * avx + avy * avy;
		mag = sqrt(mag);
		avx /= mag;
		avy /= mag;
	} else { avx = avy = 0; }

	mag = dx * dx + dy * dy;
	mag = sqrt(mag);
	dx /= mag;
	dy /= mag;


	// update dx and dy
	dx = (meMult * dx + avoidMult * avx + crowdMult * posx + copyMult * ndx);
	dy = (meMult * dy + avoidMult * avy + crowdMult * posy + copyMult * ndy);

	
	if (!(abs(dx) < 10000) || !(abs(dy) < 10000)) {
		// oh crap, something went wrong (probably div by 0)
		cout << ndx << " " << ndy << " " << dx << " " << dy << " " << avx << " " << avy << " " << posx << " " << posy << " " << endl;
		dx = dy =1;
	}
	mag = dx * dx + dy * dy;
	mag = sqrt(mag);
	dx /= mag;
	dy /= mag;

	// randomly change dz
	if (irand(1, 1000) < el)
		dz += drand(-0.02, 0.02);

	dz = dz < -0.1 ? -0.1 : dz > 0.1 ? 0.1 : dz;


	x += el * 0.1 * dx;
	y += el * 0.1 * dy;
	z += dz;

	z = z < minz ? minz : z > maxz ? maxz : z;

	// if out of the range, go somewhere new.
	if (abs(x) > 1000 || abs(y) > 1000) {
		int r = rand();
		if (r & 1) {
			r = r >> 1;
			if (r & 1) {
				y = 1000;
				r = r >> 1;
				x = - 1000 + r % 1000;
				r = r >> 1;
				dx = -1 + 2.0 * r / (RAND_MAX >> 3);
				r = r >> 1;
				dy = - 1.0 * r / (RAND_MAX >> 4);
			} else {
				y = -1000;
				r = r >> 1;
				x = - 1000 + r % 1000;
				r = r >> 1;
				dx = -1 + 2.0 * r / (RAND_MAX >> 3);
				r = r >> 1;
				dy = 1.0 * r / (RAND_MAX >> 4);
			}
		} else {
			r = r >> 1;
			if (r & 1) {
				x = 1000;
				r = r >> 1;
				y = - 1000 + r % 1000;
				r = r >> 1;
				dy = -1 + 2.0 * r / (RAND_MAX >> 3);
				r = r >> 1;
				dx = - 1.0 * r / (RAND_MAX >> 4);
			} else {
				x = -1000;
				r = r >> 1;
				y = - 1000 + r % 1000;
				r = r >> 1;
				dy = -1 + 2.0 * r / (RAND_MAX >> 3);
				r = r >> 1;
				dx = 1.0 * r / (RAND_MAX >> 4);
			}
		}

	}
}
