#include "casper.h"

#include "global.h"
#include "DrawingState.h"

float fadeSpeed = 0.0008;
float minDist = 200;


void casper::simulateUntil(unsigned long time) {
	
	if (last == 0) last = time;
	float el = time - last;
	last = time;
	if (fadeIn) { fade += fadeSpeed * el; if (fade >= 1) fadeIn = false; else return;}
	if (fadeOut) { 
		fade -= fadeSpeed * el; 
		if (fade <= 0) { 
			fadeOut = false; 
			fadeIn = true;
			x = drand(-600, 0);
			z = drand(-400, 500);
			tx = drand(max(-600, x - 200), min(0, x + 200));
			tz = drand(max(-400, z - 300), min(500, z + 300));
		}
		return;
	}
	float dx = tx - x;
	float dz = tz - z;
	float mag = dx * dx + dz * dz;
	mag = sqrt(mag);
	if (mag < 2 * el * speed) fadeOut = true;
	mag = 1 / mag;
	dx *= mag;
	dz *= mag;


	x += el * speed * dx;
	z += el * speed * dz;
	if (y > 5) dy -= 0.01 ;
	else if (y < 0) dy += 0.01;
	y += dy * el * speed;
	
}

casper::casper(float ix, float iy, float iz) : x(ix), y(iy), z(iz), fade(1), last(0), tx(0), tz(0), speed(0.04)
{
	static int id = 0;
	num = id++;
	y = drand(1, 4);
	dy = 0.1 * (irand(0, 1) == 0 ? 1 : -1);
}

casper::~casper(void)
{
}

void casper::drawSpecial(DrawingState* drst) {
	Matrix cam;
	drst->camera->getCamera(cam);
	invertMatrix(cam);
	
	
	float vect[4] = {0, 0, 0, 0};
	vect[0] = x;
	vect[1] = y;
	vect[2] = z;

	float sum = 0;
	for (int i = 0; i < 3; i++) {
		vect[i] -= cam[3][i];
		sum += vect[i] * vect[i];
	}
	sum = 1 / sqrt(sum);
	for (int i = 0; i < 3; i++) {
		vect[i] *= sum;
	}
	
	float courage = dot4(vect, cam[2]);
	if (courage < -0.94) courage = 0;
	else courage = 30 * (courage + 0.94);
	float dist = (x - cam[3][0]) * (x - cam[3][0]) + (z - cam[3][2]) * (z - cam[3][2]);
	dist = sqrt(dist);
	if (dist < minDist) courage *= -1 + 2 * (dist / minDist);

	if (drst->timeOfDay > 18) {
		glColor4f(1, 1, 1, 1);
		x = 100 + 20 * num;
		z = 100 + 20 * abs(1 - num);
	} else glColor4f(1, 1, 1, fade * courage);

	//glColor4f(1, 1, 1, fade);
	fetchTexture("ghost.tga");
	glEnable (GL_BLEND); glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glTranslatef(x, y, z);
	glRotatef(180 + atan2(cam[3][0] - x, cam[3][2] - z) * 180 / 3.14159, 0, 1, 0);
	glScalef(10, 20, 1);	
	polygoni(4, -1, 0, 0, 1, 0, 0, 1, 2, 0, -1, 2, 0);
	glDisable(GL_BLEND);
}

