#include "fireworks.h"
#include "DrawingState.h"




int particleList = 0;

using namespace fireworks_config;


Color randomColor(Color a, Color b) {
	return Color(drand(a.r, b.r), drand(a.g, b.g), drand(a.b, b.b), drand(a.a, b.a));
}


fireworks::fireworks(float ix, float iy, float iz) : last(0), x(ix), y(iy), z(iz)
{
	
}

fireworks::~fireworks(void)
{
}

fireworks::particle::particle(Color c, triple p, triple d)
:	r(c.r), 
	g(c.g), 
	b(c.b), 
	x(p.x), 
	y(p.y), 
	z(p.z), 
	dx(speedMult * d.x),
	dy(speedMult * d.y),
	dz(speedMult * d.z),
	size(pSize * drand(0.1, 0.3))
	{	}

void fireworks::particle::draw() {
	glPushMatrix();
	glColor3f(r, g, b);
	glTranslatef(x, y, z);
	glScalef(size, size, size);
	if (cfg.displayLists)	glCallList(particleList);
	else glutSolidSphere(1, 6, 4);
	glPopMatrix();
}

bool fireworks::particle::update(long dt) {
	r -= decay * dt;
	g -= decay * dt;
	b -= decay * dt;
	x += dx * dt;
	y += dy * dt;
	z += dz * dt;
	dy -= grav * speedMult * dt;
	dy = max(dy, terminal);
	return y < 0 || (r < 0 && g < 0 && b < 0);
}

fireworks::rocket::rocket(Color c, triple p, triple d, vector<particle>* par)
:	particles(par),
	r(c.r), 
	g(c.g), 
	b(c.b), 
	x(p.x), 
	y(p.y), 
	z(p.z), 
	dx(rocketSpeed * speedMult * d.x),
	dy(rocketSpeed * speedMult * d.y),
	dz(rocketSpeed * speedMult * d.z),
	size(drand(0.1, 0.3))
	{	}

void fireworks::rocket::draw() {
	glPushMatrix();
	glColor3f(r, g, b);
	glTranslatef(x, y, z);
	glScalef(size, 5 * size, size);
	glCube(1);
	glPopMatrix();
}

bool fireworks::rocket::update(long dt) {
	x += dx * dt;
	y += dy * dt;
	z += dz * dt;
	dy -= grav * speedMult * dt;
	Color f(1, 0, 0), g(0.9, 0.1, 0.1);
	for (int i = 0; i < rocketTrail; ++i)
		particles->push_back(particle(randomColor(f, g), triple(x, y, drand(z, z - dz)), triple(drand(-0.1, 0.1), drand(-0.2 * dz, -0.1 * dz), drand(-0.1, 0.1))));
	return dy < rocketBurstSpeed * speedMult;
}

void fireworks::draw(DrawingState * d)
{
	if (d->timeOfDay > 18) night = true;
	else night = false;
	if (particleList == 0) {
		particleList = glGenLists(1);
		glNewList(particleList, GL_COMPILE);
		glutSolidSphere(1, 6, 4);
		glEndList();
	}
	glTranslatef(x, y, z);
	for (vector<rocket>::iterator i = rockets.begin(); i != rockets.end(); ++i) {
		i->draw();
	}
	for (vector<particle>::iterator i = particles.begin(); i != particles.end(); ++i) {
		i->draw();
	}
	Color f(1, 1, 1), g(0.6, 0.6, 0.6);
}

void fireworks::simulateUntil(unsigned long time)
{
	if (last == 0) last = time;
	long elapsed = time - last;
	long elapsedRocket = time - lastRocket;
	last = time;
	bool shoot = false;
	if (elapsedRocket > rocketTime) {shoot = true; lastRocket = time;}

	if (shoot && rockets.size() < maxRockets && night) {
		Color f(0.5, 0, 0), g(0.6, 0.1, 0.1);
		rockets.push_back(rocket(randomColor(f, g), triple(drand(-10, 10), 0, drand(-10, 10)), triple(drand(-0.8, 0.8), drand(5, 6), drand(-0.8, 0.8)), &particles));
	}

	for (vector<rocket>::iterator i = rockets.begin(); i != rockets.end();) {
		if (i->update(elapsed)) {burst(i->x, i->y, i->z); i = rockets.erase(i); }
		else ++i;
	}

	for (vector<particle>::iterator i = particles.begin(); i != particles.end();) {
		if (i->update(elapsed)) i = particles.erase(i);
		else ++i;
	}


	
}

void fireworks::burst(float x, float y, float z) {
	// the fun part.
	Color f(0.5, 0.5, 0.5), g(1, 1, 1);
	for (int i = 0; i < burstCount; ++i) {
		double dx, dy, dz;
		double r, phi, theta;
		r = drand(0, 3);
		theta = drand(-3.14159, 3.14159);
		phi = drand(0, 3.14159);
		dz = r * cos(theta);
		r = r * sin(theta);
		dx = r * cos(phi);
		dy = r * sin(phi);
		
		particles.push_back(particle(randomColor(f, g), triple(x, y, z), triple(dx, dy, dz)));
	}
}