#include "global.h"

#include "texture.h"
#include "drawutils.h"


void gl_rotate(triple dir, triple out, triple up) {

	double mat[16] = {
		dir.x, dir.y, dir.z, 0,
		out.x, out.y, out.z, 0,
		up.x, up.y, up.z, 0,
		0, 0, 0, 1 };

		glMultMatrixd(mat);
}

void glCube(float s) {
	static int list = 0;
	if (list == 0) {
		int state = 0;
		glGetIntegerv(GL_LIST_INDEX, &state);
		if (state != 0) {
			glutSolidCube(s);
			return;
		}
		list = glGenLists(1);
		glNewList(list, GL_COMPILE);
		glutSolidCube(1);
		glEndList();
	}
	glPushMatrix();
	glScalef(s, s, s);
	glCallList(list);
	glPopMatrix();
}

void glCylinder(float s) {
	static int list = 0;
	
	if (list == 0) {
		GLUquadric* quad = gluNewQuadric();
		gluQuadricTexture(quad , GL_TRUE );
		int state = 0;
		glGetIntegerv(GL_LIST_INDEX, &state);
		if (state != 0) {
			glCube(s);
			return;
		}
		list = glGenLists(1);
		glNewList(list, GL_COMPILE);
			glEnable( GL_TEXTURE_2D );
			glRotatef(-90, 1, 0, 0);
			glTranslatef(0, 0, -0.5);
			gluCylinder(quad, 1, 1, 1, 10, 3);
			glPushMatrix();
				glScalef(1, 1, 0.1);
				gluSphere(quad, 1, 10, 6);
			glPopMatrix();
			glTranslatef(0, 0, 1);
			glScalef(1, 1, 0.1);
			gluSphere(quad, 1, 10, 6);
		glEndList();
	}


	glPushMatrix();
	glScalef(s, s, s);
	glCallList(list);
	glPopMatrix();

}

void square(float s, float d, int o, string name)
{
	glPushMatrix();
	cfg.texture(name);

	if (o == UP_Y || o == DOWN_Y) glRotatef(90, 1, 0, 0);
	if (o == UP_X || o == DOWN_X) glRotatef(90, 0, 1, 0);

	float e = 0.002;

	glTranslatef(0, 0, d);
	glScalef(s, s, s);
	glBegin(GL_QUADS);		
	if (o & 1) {
		glTexCoord2d(1 - e, 1 - e);
		glVertex3f(-1, 1, 0);
		glTexCoord2d(1 - e, 0 + e);
		glVertex3f(-1, -1, 0);
		glTexCoord2d(0 + e, 0 + e);
		glVertex3f(1, -1, 0);
		glTexCoord2d(0 + e, 1 - e);
		glVertex3f(1, 1, 0);
	} else {	
		glTexCoord2d(0, 1 - e);
		glVertex3f(-1, 1, 0);
		glTexCoord2d(1 - e, 1 - e);
		glVertex3f(1, 1, 0);
		glTexCoord2d(1 - e, 0 + e);
		glVertex3f(1, -1, 0);
		glTexCoord2d(0 + e, 0 + e);
		glVertex3f(-1, -1, 0);	
	}
	glEnd();
	glPopMatrix();

}

void csquare(float s, float d, int o, float r, float g, float b, float a)
{
	glPushMatrix();

	if (o == UP_Y || o == DOWN_Y) glRotatef(90, 1, 0, 0);
	if (o == UP_X || o == DOWN_X) glRotatef(90, 0, 1, 0);

	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(r, g, b, a);

	
	glTranslatef(0, 0, d);
	glScalef(s, s, s);
	glBegin(GL_QUADS);
	if ((o & 1)) {
		glVertex3f(-1, 1, 0);
		glVertex3f(-1, -1, 0);
		glVertex3f(1, -1, 0);
		glVertex3f(1, 1, 0);
	} else {	
		glVertex3f(-1, 1, 0);
		glVertex3f(1, 1, 0);
		glVertex3f(1, -1, 0);
		glVertex3f(-1, -1, 0);	
	}
	glEnd();
	glPopMatrix();

	glDisable(GL_BLEND);

}