#include "tree.h"

int tree::limbs[20];
int tree::bases[8];
int tree::trees[20];
vector<vector<float*> > tree::baseEnds;



using namespace tree_config;

tree::tree(void) : r(drand(0.8, 1)), g(drand(0.8, 1)), b(drand(0.8, 1))
{
}

tree::~tree(void)
{
}

void tree::draw(int id) {
	static bool made = false;
	if (!made) make();
	made = true;

	static vector<int> lists;

	while (lists.size() <= id) lists.push_back(rand() % trees::count);

	cfg.texture("treebark.jpg");
	glEnable( GL_TEXTURE_2D );
	glCallList(trees[lists[id]]);
}

tree* tree::generate(int x, int y) {
	static int count = 0;
	tree* t = new tree();
	t->id = count++;
	t->x = x;
	t->y = y;

	return t;
}

void tree::make() {
	glCube(1);
	glCylinder(1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	for (int i = 0; i < limbs::count; ++i) {
		limbs[i] = glGenLists(1);
		glNewList(limbs[i], GL_COMPILE);
		makeLimb(limbs::startLength, 0);
		glEndList();
	}

	for (int i = 0; i < bases::count; ++i) {
		bases[i] = glGenLists(1);
		vector<float*> j;
		glNewList(bases[i], GL_COMPILE_AND_EXECUTE);
		makeBase(bases::startLength, 0, j);
		glEndList();
		baseEnds.push_back(j);
	}

	for (int i = 0; i < trees::count; ++i) {
		trees[i] = glGenLists(1);
		glNewList(trees[i], GL_COMPILE);
		glPushMatrix();

		double scalev = pow(1.0 * rand() / RAND_MAX, trees::scalePower);
		scalev = rand() & 1 ? scalev : -1 * scalev;
		scalev = trees::medScale + scalev * trees::maxScaleChange;
		glScalef(scalev, scalev, scalev);
		
		int base = rand() % bases::count;
		glCallList(bases[base]);
		vector<float*> ends = baseEnds[base];
		for (int j = 0; j < ends.size(); ++j) {
			glPushMatrix();
			glMultMatrixf(ends[j]);
			glCallList(limbs[rand() % limbs::count]);
			glPopMatrix();
		}
		glPopMatrix();
		glEndList();
	}
	glPopMatrix();
}

void tree::makeBase(double length, int depth, vector<float*>& pos) {
	using namespace bases;
	
	length *= (lengthMin + (lengthMax - lengthMin) * (1.0 * rand() / RAND_MAX));
	if (length < endLength || depth > maxDepth) {float* p = new float[16]; glGetFloatv(GL_MODELVIEW_MATRIX, p); pos.push_back(p); return;}

	glPushMatrix();
	glScalef(0.1 * length, length, 0.1 * length);
	glTranslatef(0, 0.5, 0);
	glCylinder(1);	
	glPopMatrix();

	vector<double> angles;
	double angle;
	double angleLeft = 360;
	for (int i = 0; i < split - 1; ++i)
	{
		angle = angleLeft / (split - i);
		angle *= (angleVariance + 1.0 * (1 - angleVariance) * rand() / RAND_MAX);
		angles.push_back(angle);
		angleLeft -= angle;
	}
	angles.push_back(angleLeft);
	
	angle = 0;
	glPushMatrix();
	glTranslatef(0, length, 0);	
	for (int i = 0; i < split; ++i)
	{
		double currangle = downMin + (downMax - downMin) * (1.0 * rand() / RAND_MAX);
		glRotatef(angles[i], 0, 1, 0);
		glPushMatrix();
		if (i != split - 1)
			glTranslatef(0, -length + length * (branchMin + ( 1 - branchMin) * (1.0 * rand() / RAND_MAX)), 0);
		
		glRotatef(currangle, 1, 0, 0);
		makeBase(length, depth + 1, pos);
		glPopMatrix();		
	}
	glPopMatrix();
}

void tree::makeLimb(double length, int depth) {
	using namespace limbs;


	glPushMatrix();
	glScalef(0.1 * length, length, 0.1 * length);
	glTranslatef(0, 0.5, 0);
	glCylinder(1);	
	glPopMatrix();



	vector<double> angles;
	double angle;
	double angleLeft = 360;
	for (int i = 0; i < split - 1; ++i)
	{
		angle = angleLeft / split - i;
		angle *= (angleVariance + 2.0 * (1 - angleVariance) * rand() / RAND_MAX);
		angles.push_back(angle);
		angleLeft -= angle;
	}
	angles.push_back(angleLeft);
	
	glPushMatrix();
	angle = 0;
	glTranslatef(0, length, 0);
	
	
	length *= (lengthMin + (lengthMax - lengthMin) * (1.0 * rand() / RAND_MAX));
	if (length < endLength || depth > maxDepth) { /*drawLeaf();*/ glPopMatrix(); return; }
	
	

	for (int i = 0; i < split; ++i)
	{
		double currangle = downMin + (downMax - downMin) * (1.0 * rand() / RAND_MAX);
		glRotatef(angles[i], 0, 1, 0);
		glPushMatrix();
		if (i != split - 1)
			glTranslatef(0, -length + length * (branchMin + ( 1 - branchMin) * (1.0 * rand() / RAND_MAX)), 0);
		
		glRotatef(currangle, 1, 0, 0);
		makeLimb(length, depth + 1);
		glPopMatrix();		
	}
	glPopMatrix();
}

void tree::drawLeaf() {
	glPushAttrib(GL_CURRENT_BIT);
	glPushMatrix();
	glColor3f(0.1, 0.8, 0.1);
	glScalef(0.1, 3, 2);
	glTranslatef(0, 0.5, 0);
	glCube(1);	
	glPopMatrix();
	glPopAttrib();
}