#include "widow.h"
#include "global.h"
#include "DrawingState.h"

float turn = 0.002;
float speed = 0.005;
float stepSpeed = 10 * speed;
float exit_x = 200, exit_z = 100;
float widowFadeSpeed = 0.0001;

bool inRegion(float x, float y, vector<region>& bad) {
	for (vector<region>::iterator i = bad.begin(); i != bad.end(); ++i)
		if (i->in(x, y)) return true;
	return false;
}

widow::widow(vector<region>* b) : regions(b), last(0), step(0), dstep(1), x(0), z(0), dx(1), dz(0), fade(1)
{
	x = drand(-600, 0);
	z = drand(-400, 500);
	while (inRegion(x, z, *regions)) {
		x = drand(-600, 0);
		z = drand(-400, 500);
	}
	tx = drand(-600, 0);
	tz = drand(-400, 500);
	while (inRegion(tx, tz, *regions)) {
		tx = drand(-600, 0);
		tz = drand(-400, 500);
	}
}

widow::~widow(void)
{
}

void push() {glPushMatrix();}
void pop() {glPopMatrix();}



void findPath(float x, float z, float tx, float tz, vector<region>& bad, float& bx, float& bz) {
	int maximum = 30;
	float dx, dz;
	dx = tx - x;
	dz = tz - z;
	float mag = dx * dx + dz * dz;
	mag = 1/sqrt(mag);
	dx *= mag;
	dz *= mag;
	float dir = atan2(dz, dx);
	float nx = cos(dir + 3.14159 / 2), nz = sin(dir + 3.14159 / 2);

	float cdx, cdz;
	float dx_opt = 0, dz_opt = 0;
	int j_opt = 0;
	int c = 0;
	for (float i = 0; i < 0.8 * 3.14159; i += 3.14159 / 100, ++c)
	{
		float cosv = cos(i), sinv = sin(i);
		cdx = cosv * dx + sinv * nx;
		cdz = cosv * dz + sinv * nz;
		int j;
		for (j = 5; j < maximum; j += 5)
			if (inRegion(x + j * cdx, z + j * cdz, bad)) break;
	
		if (j > j_opt) {
			dx_opt = cdx;
			dz_opt = cdz;
			j_opt = j;
		}
		cdx = cosv * dx - sinv * nx;
		cdz = cosv * dz - sinv * nz;
		for (j = 5; j < maximum; j += 5)
			if (inRegion(x + j * cdx, z + j * cdz, bad)) break;
	
		if (j > j_opt) {
			dx_opt = cdx;
			dz_opt = cdz;
			j_opt = j;
		}
		if (j_opt == maximum) break;
		if (c < 20 && j_opt > maximum * .3) break;
		if (c < 50 && j_opt > maximum * .5) break;
		if (c < 75 && j_opt > maximum * .7) break;
	}

	bx = dx_opt;
	bz = dz_opt;
	mag = bx * bx + bz * bz;
	mag = mag == 0 ? 0 : 1/sqrt(mag);
	bx *= mag;
	bz *= mag;
}

void drawLeg(float angle, float fade) {
	float dist = sin(angle / 180 * 3.14159);
	float up = cos(angle / 180 * 3.14159);
	
	fetchTexture("widow-leg.tga");
	glPushMatrix();
	glTranslatef(0, up * 6, 0);
	glTranslatef(0, 10, 0);
	glRotatef(angle, 0, 0, 1);
	glTranslatef(0, -12, 0);	
	glPushMatrix();
	glScalef(1, 10, 1);
	glTranslatef(0, 0.6, 0);
	glCylinder(1);
	glPopMatrix();

	fetchTexture("widow-foot.tga");
	glTranslatef(1.5, 0.4, 0);
	glScalef(4, 1, 1);
	glRotatef(90, 0, 0, 1);
	glCylinder(1);

	glPopMatrix();

}

void arm(float angle, float fade) {
	float dist = sin(angle / 180 * 3.14159);
	float up = cos(angle / 180 * 3.14159);
	glPushMatrix();
	

	fetchTexture("widow-arm.tga");
	glTranslatef(0, up * 6, 0);
	glTranslatef(0, 10, 0);
	glRotatef(angle, 0, 0, 1);
	glTranslatef(0, -12, 0);
	
	glPushMatrix();
	glScalef(1, 10, 1);
	glTranslatef(0, 0.6, 0);
	glCylinder(1);
	glPopMatrix();

	
	fetchTexture("widow-hand.tga");
	glTranslatef(1.5, 0.4, 0);
	glScalef(4, 1, 1);
	glRotatef(90, 0, 0, 1);
	glCylinder(1);

	glPopMatrix();

}

void body(float fade) {
	fetchTexture("widow-robe.tga");
	push();
	glTranslatef(0, 17, 0);
	glScalef(4, 15, 5);
	
	glCylinder(1);
	pop();

	push();
	glTranslatef(0, 25, 0);
	glScalef(4.1, 2, 7);
	
	glCylinder(1);
	pop();
	push();
	glTranslatef(0, 27, 0);
	glScalef(1.5, 2, 1.5);
	
	glCylinder(1);
	pop();

}

void head(float fade) {
	push();
	GLUquadric* quad = gluNewQuadric();
	gluQuadricTexture(quad , GL_TRUE );
	glScalef(1, 1.2, 1);
	glRotatef(-10, 0, 1, 0);
	glRotatef(-90, 1, 0, 0);
	glRotatef(90, 0, 0, 1);
	fetchTexture("widow-head.tga");
	gluSphere(quad, 3.5, 10, 10);
	pop();

	glColor4f(1, 1, 1, 0.7);
	fetchTexture("widow-veil.tga");
	push();
	glScalef(3.7, 7, 3.7);
	glCylinder(1);
	glColor4f(1, 1, 1, 0.5);
	pop();	
}


void widow::drawSpecial(DrawingState* drst) {
	widowtime = drst->timeOfDay;
	if (fade < 0.05) return;
	glEnable (GL_BLEND); glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glColor4f(1, 1, 1, fade);



	glGetFloatv(GL_MODELVIEW_MATRIX, (float*)transform);

	for (int i = 0; i < 16; i++)
		transform[0][i] = 1;

	transform[3][0] = x;
	transform[3][2] = z;
	if (cfg.widowlegs()) {
		laX = 0; laY = 0; laZ = 0;
		lfX = 0; lfY = 20; lfZ = 0;
	} else {
		laX = 0; laY = 0; laZ = 0;
		lfX = 70; lfY = 50; lfZ = 0;
	}

	glTranslatef(x, -2, z);
	

	glRotatef(atan2(dx, dz) * 180 / 3.14159 - 90, 0, 1, 0);
	push();
	body(fade);
	pop();


	push();
	glTranslatef(0, 0, 2);
	drawLeg(dstep * step, fade);
	pop();


	push();
	glTranslatef(0, 0, -2);
	drawLeg(-dstep * step, fade);
	pop();


	push();
	glTranslatef(0, 10, -5);
	arm(dstep * step, fade);
	pop();

	push();
	glTranslatef(0, 10, 5);
	arm(-dstep * step, fade);
	pop();

	push();
	glTranslatef(0, 30.5, 0);
	head(fade);
	pop();

	glDisable(GL_BLEND);	
}



void widow::simulateUntil(unsigned long time) {
	if (!last) last = time;
	long el = time - last;
	last = time;
	if (widowtime <= 1 || widowtime >= 23) {
		tx = x = exit_x;
		tz = z = exit_z;
		fade = 0;
		return;
	}
	
	if (widowtime < 5 || widowtime > 14) {
		tx = exit_x;
		tz = exit_z;
	}
	if (fade < 1) fade += widowFadeSpeed * el;

	float d = (tx - x) * (tx - x) + (tz - z) * (tz - z);
	d = d >= 10 ? sqrt(d) : 1;
	if (d < 10 || cfg.reset) { 
		if (widowtime < 5 || widowtime > 14) { fade = fade < 0 ? 0 : fade - widowFadeSpeed * el;return; }
		else {
			tx = drand(-600, 0);
			tz = drand(-400, 500); 
			while (inRegion(tx, tz, *regions)) {
				tx = drand(-600, 0);
				tz = drand(-400, 500);
			}
		}
		if (cfg.reset) {
			x += 20;
			z += 20;
		}
		cfg.reset = false;
	}

	

	float a, b;
	findPath(x, z, tx, tz, *regions, a, b);
	
	float t = turn * el;
	
	dx = t * a + (1 - t) * dx;
	dz = t * b + (1 - t) * dz;

	float mag = dx * dx + dz * dz;
	mag = mag == 0 ? 1 : 1/sqrt(mag);
	dx *= mag;
	dz *= mag;
	
	if (d > 5) {
		x += speed * el * dx;
		z += speed * el * dz;
		step += stepSpeed * el;
	}
	float maxStep = 24;
	if (step > maxStep) { dstep *= -1; step -= 2 * maxStep; }

}

